﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[System.Serializable]
public class MoveAnimation {
	public AnimationClip clip;
	public Vector3 velocity;

	[HideInInspector]
	public float weight;

	[HideInInspector]
	public bool currentBest;

	[HideInInspector]
	public float angle, speed;

	public void Init() {
		velocity.y = 0f;
		speed = velocity.magnitude;
		angle = PlayerAnimation.HorizontalAngle(velocity);
	}
} // class

public class PlayerAnimation : MonoBehaviour {

	private Rigidbody myBody;
	public Transform rootBone, upperBodyBone;
	public float maxIdleSpeed = 0.5f, minWalkSpeed = 2f;
	public AnimationClip idle_Clip, turn_Clip, shootAdditive_Clip;
	public MoveAnimation[] moveAnimations;

	private Transform rigidBodyTransform;
	private Vector3 lastPosition = Vector3.zero;
	private Vector3 velocity = Vector3.zero;
	private Vector3 localVelocity = Vector3.zero;
	private float speed = 0f, angle = 0f, lowerBodyDeltaAngle = 0f, idleWeight = 0f;
	private Vector3 lowerBodyForward_Target = Vector3.zero;
	private Vector3 lowerBodyForward = Vector3.zero;
	private MoveAnimation bestAnimation;
	private float last_FootstepTime = 0f, last_AnimTime = 0f;

	private Animation animationComponent;

	void Awake () {
		myBody = GetComponentInParent<Rigidbody> ();
		animationComponent = GetComponent<Animation> ();

		rigidBodyTransform = myBody.transform;
		lastPosition = rigidBodyTransform.position;

		foreach(MoveAnimation moveAnimation in moveAnimations) {
			moveAnimation.Init ();
			animationComponent [moveAnimation.clip.name].layer = 1;
			animationComponent [moveAnimation.clip.name].enabled = true;
		}

		animationComponent.SyncLayer (1);

		animationComponent [idle_Clip.name].layer = 2;
		animationComponent [turn_Clip.name].layer = 3;
		animationComponent [idle_Clip.name].enabled = true;

		animationComponent [shootAdditive_Clip.name].layer = 4;
		animationComponent [shootAdditive_Clip.name].weight = 1;
		animationComponent [shootAdditive_Clip.name].speed = 0.6f;
		animationComponent [shootAdditive_Clip.name].blendMode = AnimationBlendMode.Additive;

	}

	void FixedUpdate () {
		velocity = (rigidBodyTransform.position - lastPosition) / Time.deltaTime;
		localVelocity = rigidBodyTransform.InverseTransformDirection (velocity);
		localVelocity.y = 0f;
		speed = localVelocity.magnitude;
		angle = HorizontalAngle (localVelocity);

		lastPosition = rigidBodyTransform.position;
	}

	void Update () {
		idleWeight = Mathf.Lerp (idleWeight, Mathf.InverseLerp (minWalkSpeed, maxIdleSpeed, speed),
			Time.deltaTime * 10f);

		animationComponent [idle_Clip.name].weight = idleWeight;

		if (speed > 0) {
			float smallestDiff = Mathf.Infinity;

			foreach (MoveAnimation moveAnim in moveAnimations) {
				float angleDiff = Mathf.Abs (Mathf.DeltaAngle (angle, moveAnim.angle));
				float speedDiff = Mathf.Abs (speed - moveAnim.speed);
				float diff = angleDiff + speedDiff;

				if (moveAnim == bestAnimation)
					diff *= 0.9f;

				if (diff < smallestDiff) {
					bestAnimation = moveAnim;
					smallestDiff = diff;
				}
					
			}

			animationComponent.CrossFade (bestAnimation.clip.name);

		} else {
			bestAnimation = null;
		}

		if (lowerBodyForward != lowerBodyForward_Target && idleWeight >= 0.9f) {
			animationComponent.CrossFade (turn_Clip.name, 0.05f);
		}

		if (bestAnimation != null && idleWeight < 0.9f) {
			float newAnimTime = Mathf.Repeat (animationComponent [bestAnimation.clip.name].normalizedTime
			                    * 2f + 0.1f, 1f);

			if (newAnimTime < last_AnimTime) {
				if (Time.time > last_FootstepTime + 0.1f) {
					last_FootstepTime = Time.time;
				}
			}

			last_AnimTime = newAnimTime;
		}

		if (Input.GetMouseButtonDown (0)) {
			OnStartFire ();
		}

		if (Input.GetMouseButtonUp (0)) {
			OnEndFire ();
		}
	}

	void LateUpdate() {
		float idle = Mathf.InverseLerp (minWalkSpeed, maxIdleSpeed, speed);

		if (idle < 1) {
			Vector3 animatedLocalVelocity = Vector3.zero;
			foreach (MoveAnimation moveAnim in moveAnimations) {
				if (animationComponent [moveAnim.clip.name].weight == 0)
					continue;

				if (Vector3.Dot (moveAnim.velocity, localVelocity) == 0)
					continue;

				animatedLocalVelocity += moveAnim.velocity *
				animationComponent [moveAnim.clip.name].weight;
			}

			float lowerBodyDeltaAngleTarget = Mathf.DeltaAngle (
              HorizontalAngle (rigidBodyTransform.rotation * animatedLocalVelocity),
              HorizontalAngle (velocity)
      		);

			lowerBodyDeltaAngle = Mathf.LerpAngle (lowerBodyDeltaAngle, lowerBodyDeltaAngleTarget,
				Time.deltaTime * 10f);

			lowerBodyForward_Target = rigidBodyTransform.forward;
			lowerBodyForward = Quaternion.Euler (0f, lowerBodyDeltaAngle, 0f) *
				lowerBodyForward_Target;

		} else {
			lowerBodyForward = Vector3.RotateTowards (lowerBodyForward, lowerBodyForward_Target,
				Time.deltaTime * 520f * Mathf.Deg2Rad, 1f);

			lowerBodyDeltaAngle = Mathf.DeltaAngle (HorizontalAngle (rigidBodyTransform.forward),
				HorizontalAngle (lowerBodyForward));

			if (Mathf.Abs (lowerBodyDeltaAngle) > 80f) {
				lowerBodyForward_Target = rigidBodyTransform.forward;
			}

		}

		Quaternion lowerBodyDeltaRotation = Quaternion.Euler (0f, lowerBodyDeltaAngle, 0f);

		rootBone.rotation = lowerBodyDeltaRotation * rootBone.rotation;

		upperBodyBone.rotation = Quaternion.Inverse (lowerBodyDeltaRotation) * upperBodyBone.rotation;

	}

	void OnStartFire () {
		animationComponent [shootAdditive_Clip.name].enabled = true;
	}

	void OnEndFire () {
		animationComponent [shootAdditive_Clip.name].enabled = false;
	}

	public static float HorizontalAngle(Vector3 direction) {
		return Mathf.Atan2 (direction.x, direction.z) * Mathf.Rad2Deg;
	}

} // class












































